from typing import Any
from typing import Iterable
from typing import Mapping
from typing import Optional

import pluggy


class Configuration:
    def __init__(
        self, options: Mapping[Any, Any], plugin_manager: pluggy.PluginManager
    ) -> None:
        self.options = options
        self.plugin_manager = plugin_manager

    def get_option(self, option: str) -> Optional[Any]:
        """
        Fetches a command line option from the config.
        Config options need to be dynamic because arbitrary third party plugins can add their own options
        """
        return self.options.get(option)

    @property
    def target(self) -> str:
        return self.get_option("target")

    @property
    def port_range(self) -> Iterable[int]:
        return self.get_option("port_range")

    @property
    def quick(self) -> bool:
        return self.get_option("quick")

    @property
    def random(self) -> bool:
        return self.get_option("random")

    @property
    def thread_count(self) -> int:
        return self.get_option("thread_count")

    def __repr__(self) -> str:
        return f"Configuration=(options={repr(self.options)}"



"""Top-level package for sonic."""

__author__ = """Simon Kerr"""
__email__ = "jackofspaces@gmail.com"
__version__ = "0.1.0"

import colorama
import pluggy

from .configuration import Configuration
from .hooks import hookspec

# TODO: consider moving this out of init.py; is it bad practice?
colorama.init()
plugin_manager = pluggy.PluginManager("sonic")
plugin_manager.add_hookspecs(hookspec)

sonic_hookspec = hookspec.sonic_hookspec
sonic_hookimpl = hookspec.sonic_hookimpl

__all__ = [
    "Configuration",
    "plugin_manager",
    "hookspec",
    "sonic_hookimpl",
    "sonic_hookspec",
]



"""Console script for sonic."""
import argparse
import sys

from colorama import Fore
from pluggy import PluginManager

from zonic import Configuration
from zonic import plugin_manager


def sonic_add_options(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "--target",
        action="store",
        help="The target host to inspect available ports of.",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Run sonic in verbose mode",
        dest="verbose",
    )
    parser.add_argument(
        "-q",
        "--quick",
        action="store_true",
        help="Scan only the top 100 popular open ports",
        dest="quick",
    )
    parser.add_argument(
        "-r",
        "--random",
        action="store_true",
        help="Scan ports in the port range randomly, not sequentially.",
        dest="random",
    )
    parser.add_argument(
        "-tc",
        "--thread-count",
        action="store",
        type=int,
        default=5_000,
        help="How many threads should scanning use, this depends heavily on available memory, tweak accordingly.",
        dest="thread_count",
    )
    parser.add_argument(
        "-pr",
        "--ports",
        action="store",
        type=int,
        default=range(65536),
        help="Specify the port range to perform a scan on.",
        dest="port_range",
    )
    parser.add_argument(
        "-wv",
        "--write-vulnerable",
        action="store_true",
        default=False,
        help="If vulnerable ports are detected; write them to csv in the cwd",
        dest="write_vulnerable",
    )


def main():
    welcome()
    parser = argparse.ArgumentParser()
    plugin_manager.register(__file__, "sonic_main")
    plugin_options = plugin_manager.hook.sonic_add_options(parser=parser)
    return 0


def _register_core_plugins(
    config: Configuration, plugin_manager: PluginManager
) -> None:
    from zonic.core.sonic_core import SonicCorePlugin
    from zonic.core.sonic_csv import SonicCSVPlugin

    core_plugin = SonicCorePlugin(config, plugin_manager)
    csv_plugin = SonicCSVPlugin(config, plugin_manager)
    plugin_manager.register(plugin=core_plugin, name=core_plugin.name)
    plugin_manager.register(plugin=csv_plugin, name=csv_plugin.name)


def welcome() -> None:
    print(
        fr"""
================================ {Fore.LIGHTBLUE_EX}
   _____             _
  / ___/____  ____  (_)____
  \__ \/ __ \/ __ \/ / ___/
 ___/ / /_/ / / / / / /__
/____/\____/_/ /_/_/\___/

{Fore.RESET} =================== {Fore.WHITE}
  sonic: Powerful python port scanner, https://github.com/symonk/sonic
    """,
        Fore.RESET,
    )


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



import socket
from concurrent.futures import ThreadPoolExecutor
from concurrent.futures import as_completed
from queue import Queue
from random import shuffle
from typing import Iterable
from typing import List
from typing import Optional

from halo import Halo

from zonic.decorators import stdout_scan_time


class PortScanner:
    def __init__(
        self,
        target: str,
        port_range: Iterable[int],
        quick: bool,
        random: bool,
        thread_count: int,
    ) -> None:
        self.target = socket.gethostbyname(target)
        self.port_range = port_range
        self.quick = quick
        self.random = random
        self.thread_count = thread_count
        self.ports_queue: Queue = Queue()

    @stdout_scan_time
    def attack(self, randomize: bool) -> List[Optional[int]]:
        executor = ThreadPoolExecutor(max_workers=self.thread_count)
        futures = []
        print(
            f"**** Attack scan strategy is: {'random' if randomize else 'sequential'}"
        )
        if randomize:
            shuffle(self.port_range)

        with Halo(text="***** Port scanning in progress...", color="blue"):
            for port in self.port_range:
                futures.append(executor.submit(self.scan_port, port))
            for future in as_completed(futures):
                result = future.result()
                if result is not None:
                    self.ports_queue.put(result)
            return list(self.ports_queue.queue)

    def scan_port(self, port: int) -> Optional[int]:
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            result = s.connect_ex((self.target, port))
            if not result:
                return port
        return None



import functools
import time


def stdout_separated(f):
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        print("------------------------------------------")
        return f(*args, **kwargs)

    return wrapper


def stdout_scan_time(f):
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        start = time.time()
        result = f(*args, **kwargs)
        print(f"**** Port scanning took: {int(time.time() - start)} seconds.")
        return result

    return wrapper



#!/usr/bin/env python

"""The setup script."""
from typing import List

from setuptools import find_packages
from setuptools import setup

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements: List = ["colorama", "pluggy", "halo"]

setup_requirements = ["pytest-runner"]

test_requirements = ["pytest>=3"]

setup(
    author="Simon Kerr",
    author_email="jackofspaces@gmail.com",
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.8",
    ],
    description="Port scanning made easy with python",
    entry_points={"console_scripts": ["zonic=zonic.main:main"]},
    install_requires=requirements,
    license="MIT license",
    long_description="complete me later",
    include_package_data=True,
    keywords="zonic",
    name="zonic",
    packages=find_packages(include=["zonic", "zonic.*"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/symonk/zonic",
    version="0.1.0",
    zip_safe=False,
)
